package com.bweb.controller.goods;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.security.SessionObject;
import com.hfkj.common.security.UserCenter;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.entity.*;
import com.hfkj.model.goods.GoodsModel;
import com.hfkj.model.goods.GoodsModelSpecs;
import com.hfkj.model.ResponseData;
import com.hfkj.model.SecUserSessionObject;
import com.hfkj.service.goods.*;
import com.hfkj.sysenum.SecUserObjectTypeEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Controller
@RequestMapping(value="/goods")
@Api(value="商品信息")
public class GoodsController {

    private static final Logger log = LoggerFactory.getLogger(GoodsController.class);

    @Resource
    private GoodsMsgService goodsMsgService;

    @Resource
    private GoodsBrandService goodsBrandService;

    @Resource
    private GoodsSpecsService goodsSpecsService;

    @Resource
    private GoodsTypeService goodsTypeService;

    @Resource
    private GoodsVpdService goodsVpdService;

    @Resource
    private UserCenter userCenter;

    @RequestMapping(value="/editGoodsMsg",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "编辑商品信息")
    public ResponseData editGoodsMsg(@RequestBody GoodsMsg body, HttpServletRequest request) {

        SessionObject sessionObject = userCenter.getSessionObject(request);
        SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();

        if (userModel.getAccount().getObjectType() != SecUserObjectTypeEnum.type2.getCode()) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.ROLE_NOT_PERMISSIONS, "");
        }

        if (body == null
                || body.getGoodsBrand() == null
                || body.getGoodsType() == null
                || body.getType() == null
                || StringUtils.isBlank(body.getListImg())
                || StringUtils.isBlank(body.getBannerImg())
                || StringUtils.isBlank(body.getDetailImg())
                || StringUtils.isBlank(body.getTitle())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
        }

        GoodsMsg goodsMsg;

        if (body.getId() != null) {
            // 查询品牌
            goodsMsg = goodsMsgService.queryDetail(body.getId());
            if (goodsMsg == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.CONTENT_NOT_FOUND, "");
            }
        } else {
            goodsMsg = new GoodsMsg();
            goodsMsg.setCreateTime(new Date());
        }

        goodsMsg.setUpdateTime(new Date());
        goodsMsg.setGoodsBrandName(goodsBrandService.findGoodsBrandName(body.getGoodsBrand()));
        goodsMsg.setGoodsTypeName(goodsTypeService.findGoodsType(body.getGoodsType()));
        goodsMsg.setStatus(2);
        goodsMsg.setTitle(body.getTitle());
        goodsMsg.setMerId(userModel.getAccount().getId());
        goodsMsg.setMerName(userModel.getAccount().getUserName());

        if (body.getId() != null) {
            goodsMsgService.update(goodsMsg);
        } else {
            goodsMsgService.create(goodsMsg);
        }

        return ResponseMsgUtil.success("操作成功");
    }

    @RequestMapping(value="/editGoodsSpecs",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "编辑商品规格")
    public ResponseData editGoodsSpecs(@RequestBody GoodsModelSpecs body, HttpServletRequest request) {

        SessionObject sessionObject = userCenter.getSessionObject(request);
        SecUserSessionObject userModel = (SecUserSessionObject) sessionObject.getObject();

        if (userModel.getAccount().getObjectType() != SecUserObjectTypeEnum.type2.getCode()) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.ROLE_NOT_PERMISSIONS, "");
        }

        if (body == null
                || body.getGoodsId() == null
                || body.getOriginalPrice() == null
                || body.getSalePrice() == null
                || body.getPurLimit() == null
                || body.getStock() == null
                || StringUtils.isBlank(body.getBannerImg())
                || StringUtils.isBlank(body.getShowImg())
                || StringUtils.isBlank(body.getName())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
        }

        GoodsMsg goodsMsg = goodsMsgService.queryDetail(body.getGoodsId());

        if (goodsMsg == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.CONTENT_NOT_FOUND, "");
        }



        GoodsSpecs goodsSpecs;
        GoodsVpd goodsVpd;


        if (body.getId() != null) {
            // 查询规格
            goodsSpecs = goodsSpecsService.queryDetail(body.getId());
            if (goodsSpecs == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.CONTENT_NOT_FOUND, "");
            }
        } else {
            goodsSpecs = new GoodsSpecs();
            goodsSpecs.setCreateTime(new Date());
        }

        goodsSpecs.setUpdateTime(new Date());
        goodsSpecs.setGoodsId(body.getGoodsId());
        goodsSpecs.setOriginalPrice(body.getOriginalPrice());
        goodsSpecs.setSalePrice(body.getSalePrice());
        goodsSpecs.setPurLimit(body.getPurLimit());
        goodsSpecs.setStock(body.getStock());
        goodsSpecs.setBannerImg(body.getBannerImg());
        goodsSpecs.setShowImg(body.getShowImg());
        goodsSpecs.setName(body.getName());
        goodsSpecs.setStatus(1);


        if (goodsSpecs.getId() != null) {
            goodsSpecsService.update(goodsSpecs);
        } else {
            goodsSpecsService.create(goodsSpecs);
        }

        // 判断当前商品是否是虚拟商品
        if(goodsMsg.getType() == 2) {
            // 判断必填项
            if ( body.getSpecsId() == null
                    || body.getRecycleDay() == null
                    || body.getType() == null
                    || body.getSalesEndTime() == null
                    || body.getSource() == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 查询虚拟产品
            goodsVpd = goodsVpdService.queryDetailBySpecsId(goodsSpecs.getId());

            if (goodsVpd == null) {
                goodsVpd = new GoodsVpd();
                goodsVpd.setCreateTime(new Date());
            }

            goodsVpd.setSpecsId(goodsSpecs.getId());
            goodsVpd.setRecycleDay(body.getRecycleDay());
            goodsVpd.setType(body.getType());
            goodsVpd.setJumpType(body.getJumpType());
            goodsVpd.setJumpUrl(body.getJumpUrl());
            goodsVpd.setAppidId(body.getAppidId());
            goodsVpd.setSalesEndTime(body.getSalesEndTime());
            goodsVpd.setSource(body.getSource());
            goodsVpd.setSalesEndTime(body.getSalesEndTime());
            goodsVpd.setUpdateTime(new Date());

            if (goodsVpd.getId() != null) {
                goodsVpdService.update(goodsVpd);
            } else {
                goodsVpdService.create(goodsVpd);
            }
        }

        return ResponseMsgUtil.success("操作成功");
    }

    @RequestMapping(value="/getListGoods",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询列表")
    public ResponseData getListGoods(@RequestParam(value = "title" , required = false) String title,
                                     @RequestParam(value = "goodsType" , required = false) Long goodsType,
                                     @RequestParam(value = "goodsBrand" , required = false) Long goodsBrand,
                                     @RequestParam(value = "type" , required = false) Integer type,
                                     @RequestParam(value = "status" , required = false) Integer status,
                                     @RequestParam(value = "pageNum" , required = true) Integer pageNum,
                                     @RequestParam(value = "pageSize" , required = true) Integer pageSize) {
        try {

            Map<String , Object> map = new HashMap<>();
            map.put("title", title);
            map.put("goodsType", goodsType);
            map.put("goodsBrand", goodsBrand);
            map.put("type", type);
            map.put("status", status);

            PageHelper.startPage(pageNum, pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(goodsMsgService.getList(map)));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value="/getGoodsDetail",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询商品详情")
    public ResponseData getGoodsDetail(@RequestParam(value = "id" , required = false) Long id) {
        try {


            GoodsModel goodsModel = new GoodsModel();
            GoodsMsg goodsMsg = goodsMsgService.queryDetail(id);
            BeanUtils.copyProperties(goodsMsg, goodsModel);

            Map<String , Object> map = new HashMap<>();
            map.put("goodsId" , id);

            List<GoodsSpecs> goodsSpecs = goodsSpecsService.getList(map);

            List<GoodsModelSpecs> goodsModelSpecs = new ArrayList<>();

            for (GoodsSpecs goodsSpec : goodsSpecs) {
                GoodsModelSpecs goodsModelSpec = new GoodsModelSpecs();
                BeanUtils.copyProperties(goodsSpec, goodsModelSpec);
                GoodsVpd goodsVpd = goodsVpdService.queryDetailBySpecsId(goodsSpec.getId());
                if (goodsVpd != null) {
                    BeanUtils.copyProperties(goodsVpd, goodsModelSpec);
                }
                goodsModelSpecs.add(goodsModelSpec);
            }

            return ResponseMsgUtil.success(goodsModelSpecs);

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
